package me.Demon.DemonLevels.api;

import com.yaohun.playermanage.PlayerManage;
import me.Demon.DemonLevels.Main;
import me.Demon.DemonLevels.data.PlayerData;
import me.Demon.DemonLevels.event.ChangExpEvant;
import me.Demon.DemonLevels.event.UpLevelEvant;
import me.Demon.DemonLevels.listener.MobDeath;
import me.Demon.DemonLevels.util.ScriptEngineAPI;
import org.bukkit.Bukkit;
import org.bukkit.entity.Player;
import org.bukkit.scheduler.BukkitRunnable;
import org.serverct.ersha.jd.AttributeAPI;

import java.math.BigInteger;
import java.util.List;
import java.util.Map;

public abstract class DLevelAPI {

    /*
    * 获取离线玩家的等级
    * */
    public static double getOffPlayer_Level(String playName){
        // 通过玩家名字获取玩家的UUID
        String uuid = PlayerManage.getNameToUuid(playName);
        // 检查UUID的长度，如果长度小于等于16，则认为UUID无效，返回0
        if (uuid.length() <= 16) {
            return 0;
        }
        // 根据UUID获取玩家的数据
        PlayerData playerData = Main.serverManage.getPlayerData(uuid);
        // 返回玩家的等级
        return playerData.getLevel();
    }
    /*
    * 获取玩家当前经验值
    * */
    public static BigInteger getPlayerExp(Player p){
        if(p == null){
            return new BigInteger("0");
        }
        String uuid = p.getUniqueId().toString();
        PlayerData playerData = Main.serverManage.getPlayerData(uuid);
        return playerData.getExp();
    }

    /*
    * 获取玩家总计经验
    * */
    public static BigInteger getPlayerTotalExp(Player p){
        if(p == null){
            return new BigInteger("0");
        }
        String uuid = p.getUniqueId().toString();
        PlayerData playerData = Main.serverManage.getPlayerData(uuid);
        return playerData.getTotalExp();
    }

    /*
    * 获取玩家当前等级
    * */
    public static int getPlayerLevel(Player p){
        if(p == null){
            return -1;
        }
        String uuid = p.getUniqueId().toString();
        PlayerData playerData = Main.serverManage.getPlayerData(uuid);
        return playerData.getLevel();
    }

    /*
    * 获取玩家当前等级所需经验
    * */
    public static BigInteger getLevelNeedExp(int level){
        if(level < 0){
            return new BigInteger("99999999");
        }
        BigInteger defaultExp = new BigInteger("1000"); // 默认所需经验 1000
        List<Map.Entry<String,Integer>> list = Main.configYml.getLevelNeedExpFormat();
        for (int i = 0; i < list.size(); i++) {
            int levelKey = list.get(i).getValue();
            if(level >= levelKey) {
                String expFormat = list.get(i).getKey();
                expFormat = expFormat.replace("%level%", String.valueOf(level));
                return ScriptEngineAPI.getExpFormat(expFormat);
            }
        }
        return defaultExp;
    }

    /*
    * 获取玩家当前升级所需经验
    * */
    public static BigInteger getPlayerExpToLevel(Player p){
        if(p == null){
            return new BigInteger("99999999");
        }
        String uuid = p.getUniqueId().toString();
        PlayerData playerData = Main.serverManage.getPlayerData(uuid);
        return getLevelNeedExp(playerData.getLevel());
    }

    /*
    * 伪装击杀暴徒获得经验
    * */
    public static void disguiseKillsExp(Player p,int exp){
        // 获取服务器当前经验倍率
        double serverDouble = Main.serverManage.getExpMultiplicity();
        // 获取玩家经验加成属性
        double att_expAddition = 1;
        if(AttributeAPI.getAttrData(p) != null) {
            if (AttributeAPI.getAttrData(p).getAttributeValue().get("经验加成") != null) {
                att_expAddition = 1 + (AttributeAPI.getAttrData(p).getAttributeValue().get("经验加成") / 100);
            }
        }
        exp = (int) (exp * serverDouble * att_expAddition);
        double magnificationDisplay = (serverDouble + att_expAddition) - 2; // 获取玩家当前倍率
        int magnificationDisplayString = (int) (magnificationDisplay * 100);
        // 给予玩家经验
        addExp(p,exp);
        MobDeath.experienceGainTips(p,exp,magnificationDisplayString);
    }

    /*
    * 给予玩家经验值
    * */
    public static void addExp(Player p,int exp){
        if(p == null){return;}
        if(exp <= 0){return;}
        //监听器-经验增加
        ChangExpEvant event = new ChangExpEvant(p,exp);
        Bukkit.getPluginManager().callEvent(event);
        // 获取玩家的PlayerData
        String uuid = p.getUniqueId().toString();
        PlayerData playerData = Main.serverManage.getPlayerData(uuid);
        BigInteger addExp = new BigInteger(String.valueOf(exp));
        playerData.setTotalExp(playerData.getTotalExp().add(addExp)); // 增加玩家总经验
        int level = playerData.getLevel(); // 获取玩家当前等级
        BigInteger needExp = DLevelAPI.getLevelNeedExp(level); // 获取玩家升级所需经验
        addExp = playerData.getExp().add(addExp); // 增加玩家当前经验
        // 循环检查是否有足够的经验来升级
        while (addExp.compareTo(needExp) >= 0) {
            // 检查level是否小于0，如果是则退出循环
            if (level < 0) {
                break;
            }
            // 从addExp中减去升级所需经验
            addExp = addExp.subtract(needExp);
            // 增加玩家等级
            level++;
            // 更新所需经验值以升级到下一级
            needExp = DLevelAPI.getLevelNeedExp(level);
            // 保存玩家数据
            playerData.savePlayerData();
            // 监听事件 - 升级等级
            UpLevelEvant upLevelEvant = new UpLevelEvant(p,level);
            Bukkit.getPluginManager().callEvent(upLevelEvant);
        }
        playerData.setExp(addExp);
        playerData.setLevel(level);
        updateThePlayerExperienceBar(p,playerData);
    }

    /*
    * 扣除玩家的经验
    * */
    public static void takeExp(Player p,int exp){
        if(p == null){return;}
        if(exp <= 0){return;}
        // 获取玩家的PlayerData
        String uuid = p.getUniqueId().toString();
        PlayerData playerData = Main.serverManage.getPlayerData(uuid);
        BigInteger takeExp = new BigInteger(String.valueOf(exp));
        BigInteger currentExp = playerData.getTotalExp();
        if (currentExp.compareTo(takeExp) < 0) {
            // 如果当前总经验少于要移除的经验，将总经验设置为0
            playerData.setTotalExp(BigInteger.ZERO);
            takeExp = currentExp;
        } else {
            // 否则减少玩家总经验
            playerData.setTotalExp(currentExp.subtract(takeExp));
        }
        int level = playerData.getLevel();
        BigInteger needExp = DLevelAPI.getLevelNeedExp(level);
        BigInteger currentLevelExp = playerData.getExp();
        while (takeExp.compareTo(BigInteger.ZERO) > 0 && level >= 0) {
            if (currentLevelExp.compareTo(takeExp) >= 0) {
                // 当前等级的经验足够减少
                currentLevelExp = currentLevelExp.subtract(takeExp);
                takeExp = BigInteger.ZERO;
            } else {
                // 当前等级的经验不足以完全减少，需要降级
                takeExp = takeExp.subtract(currentLevelExp);
                level--;
                if (level >= 0) {
                    needExp = DLevelAPI.getLevelNeedExp(level);
                    currentLevelExp = needExp.subtract(BigInteger.ONE); // 假设刚刚升级所剩经验为 needExp - 1
                } else {
                    currentLevelExp = BigInteger.ZERO;
                    break;
                }
            }
        }
        playerData.setExp(currentLevelExp);
        playerData.setLevel(level);
        playerData.savePlayerData();
        updateThePlayerExperienceBar(p,playerData);
    }

    public static void setTotalExp(Player p,BigInteger exp){
        if(p == null){return;}
        // 获取玩家的PlayerData
        String uuid = p.getUniqueId().toString();
        PlayerData playerData = Main.serverManage.getPlayerData(uuid);
        BigInteger totalExp = new BigInteger(String.valueOf(exp));
        playerData.setTotalExp(totalExp);
        int level = 1;
        // 获取当前等级1所需的经验
        BigInteger needExp = DLevelAPI.getLevelNeedExp(level);
        // 循环检查总经验是否足够升级
        while (totalExp.compareTo(needExp) >= 0) {
            // 从总经验中减去当前等级所需的经验
            totalExp = totalExp.subtract(needExp);
            // 升级玩家等级
            level++;
            // 获取下一个等级所需的经验
            needExp = DLevelAPI.getLevelNeedExp(level);
            // 如果等级小于0，退出循环（虽然在实际中等级通常不会小于0）
            if (level < 0) {
                break;
            }
        }
        // 更新玩家的数据
        playerData.setExp(totalExp);
        playerData.setLevel(level);
        updateThePlayerExperienceBar(p,playerData);
        playerData.savePlayerData();
    }

    public static void setPlayerLevel(Player p,int level){
        if(p == null){return;}
        if(level <= 0){return;}
        // 获取玩家的PlayerData
        String uuid = p.getUniqueId().toString();
        PlayerData playerData = Main.serverManage.getPlayerData(uuid);
        playerData.setLevel(level);
        // 初始化经验值
        BigInteger defaultExp = BigInteger.ZERO;
        // 循环计算到当前等级所需的经验
        for (int i = 1; i < level; i++) {
            // 获取当前等级所需经验
            BigInteger currentLevelNeedExp = DLevelAPI.getLevelNeedExp(i);
            defaultExp = defaultExp.add(currentLevelNeedExp);
        }
        // 计算并设置玩家的经验值
        // 由于经验值是从上一级到当前级别的剩余经验
        BigInteger totalExp = defaultExp;
        // 设置玩家的总经验
        setTotalExp(p,totalExp);
    }

    public static void updateThePlayerExperienceBar(Player p,PlayerData playerData){
        new BukkitRunnable() {
            public void run() {
                int pLevel = playerData.getLevel();
                p.setLevel(pLevel);
                BigInteger playerExp = playerData.getExp();
                BigInteger needLevelExp = DLevelAPI.getLevelNeedExp(pLevel);
                float finalExp = playerExp.floatValue() / needLevelExp.floatValue();
                p.setExp((finalExp > 1.0F) ? 1.0F : (Math.max(finalExp, 0.0F)));
                cancel();
            }
        }.runTaskLater(Main.plugin, 2L);
    }

    public static String getRankTitle(int level){
        return Main.configYml.getRankTitle(level);
    }
}
